@@ -10,7 +10,7 @@ require Exporter;
 use base 'Exporter';
 use Keyword::Pluggable;
 
-$VERSION = '1.04_3';
+$VERSION = '1.01';
 @EXPORT = qw(sql);
 @EXPORT_OK = qw(union intersect except subselect);
 %EXPORT_TAGS = (all => [@EXPORT, @EXPORT_OK]);
@@ -31,13 +31,6 @@ sub optree_version
 	return 2;
 }
 
-sub lexify
-{
-	my ( $text, $insert ) = @_;
-	$insert .= 'sub ' if $$text =~ /^\s*\{/;
-	substr($$text, 0, 0, $insert);
-}
-
 sub import
 {
 	my $pkg = caller;
@@ -73,25 +66,17 @@ sub import
 	my $iprefix = '__' . $dbh . '_execute_perlish';
 	$iprefix =~ s/\W//g;
 
-	for (
-		[fetch  => " $dbh, q(fetch), "],
-		[select => " $dbh, q(fetch), "],
-		[update => " $dbh, q(update), "],
-		[delete => " $dbh, q(delete), "],
-	) {	
-		my ($name, $code) = @$_;
-		Keyword::Pluggable::define
-			keyword    => $prefix . '_' . $name, 
-			code       => sub { lexify( $_[0], $iprefix.$code ) },
-			expression => 1,
-			package    => $pkg
-		;
-	}
 	Keyword::Pluggable::define
-		keyword    => $prefix . '_insert',
-		code       => $iprefix . "_insert $dbh, ",
+		keyword    => $prefix . '_' . $$_[0], 
+		code       => $iprefix . $$_[1],
 		expression => 1,
 		package    => $pkg
+	for
+		[fetch  => " $dbh, q(fetch),  sub "],
+		[select => " $dbh, q(fetch),  sub "],
+		[update => " $dbh, q(update), sub "],
+		[delete => " $dbh, q(delete), sub "],
+		[insert => "_insert $dbh, "],
 	;
 
 	{
@@ -329,8 +314,6 @@ sub gen_sql
 	my ($sub, $operation, %args) = @_;
 
 	$args{quirks} = $non_object_quirks unless $args{quirks};
-	$args{inline} //= 1;
-
 	my $S = DBIx::Perlish::Parse::init(%args, operation => $operation);
 	DBIx::Perlish::Parse::parse_sub($S, $sub);
 	my $sql = "";
@@ -450,15 +433,11 @@ sub gen_sql
 		die "unfiltered $operation is dangerous: use exec if you want it\n";
 	}
 
-	my $use_rownum = $args{flavor} && $args{flavor} eq "oracle";
-
-	unless ($use_rownum) {
-		if ($S->{limit}) {
-			$sql .= " limit $S->{limit}";
-		}
-		if ($S->{offset}) {
-			$sql .= " offset $S->{offset}";
-		}
+	if ($S->{limit}) {
+		$sql .= " limit $S->{limit}";
+	}
+	if ($S->{offset}) {
+		$sql .= " offset $S->{offset}";
 	}
 	my $v = $S->{set_values};
 	push @$v, @{$S->{ret_values}};
@@ -471,13 +450,6 @@ sub gen_sql
 	}
 	$sql =~ s/\s+$//;
 
-	if ( $use_rownum && ( $S->{limit} || $S->{offset} )) {
-		my @p;
-		push @p, "ROWNUM > " . $S->{offset} if $S->{offset};
-		push @p, "ROWNUM <= " . ($S->{limit} + ($S->{offset} // 0)) if $S->{limit};
-		$sql = "select * from ($sql) where " . join(' and ', @p);
-	}
-
 	return ($sql, $v, $nret, %flags);
 }
 
@@ -910,7 +882,7 @@ the function might throw any of the exceptions thrown by DBI.
 
 The C<db_insert {}> function is exported by default.
 
-=head3 subselect {}
+=head3 subselect()
 
 This call, formerly known as as internal form of C<db_fetch>, 
 is basically an SQL SELECT statement. See L</Subqueries>.
