@@ -10,8 +10,6 @@ use B;
 use Carp;
 use Devel::Caller qw(caller_cv);
 
-sub _o($) { ref($_[0]) . ( $_[0]->can('name') ? ("." . $_[0]->name) : '' ) }
-
 sub bailout
 {
 	my ($S, @rest) = @_;
@@ -113,7 +111,7 @@ sub want_pushmark_or_padrange
 {
 	my ($S, $op) = @_;
 	unless (is_pushmark_or_padrange($op)) {
-		bailout $S, "want op pushmark or op padrange, got " . _o $op;
+		bailout $S, "want op pushmark or op padrange";
 	}
 }
 
@@ -274,12 +272,12 @@ sub aux_get_padsv
 						last;
 					}
 				}
-				goto DEFAULT_PADLIST unless $store->{outer_padlist};
+				bailout $store->{S}, "cannot refer to an outer padlist ".$store->{inner}->outid
+					unless $store->{outer_padlist};
 				$store->{outer_padlist_array} = [$store->{outer_padlist}->[1]->ARRAY];
 			}
 			$store->{padlist}->[$index] = $store->{outer_padlist_array}->[ $padname->PARENT_PAD_INDEX ];
 		} else {
-		DEFAULT_PADLIST:
 			$store->{padlist}->[$index] = $store->{orig_pads}->ARRAYelt($index);
 		}
 	}
@@ -298,16 +296,11 @@ sub parse_multideref
  	ITEMS: while ( @items ) {
  		my $actions = shift @items;
 
-		my ($ref, $reftype);
-		my $sv = shift(@items) or return undef;
+		my $ref;
+		my $sv = shift(@items) or bailout $S, "unexpected empty multideref";
 
-		while ( @items ) {
-			my $ptr = shift @items;
+		while ( my $ptr = shift @items ) {
  			my $access  = $actions & B::MDEREF_ACTION_MASK();
-			if ( $access == B::MDEREF_reload() ) { # XXX
-				$actions = $sv;
-				next;
-			}
 			unless ($ref) {
  				if (
  					$access == B::MDEREF_HV_padhv_helem() ||
@@ -332,52 +325,32 @@ sub parse_multideref
 				) {
 					bailout_multiref_vivify $S unless ref($sv);
 					$ref = $sv->AV->object_2svref;
-				} elsif (
-					$access == B::MDEREF_AV_gvsv_vivify_rv2av_aelem() ||
-					$access == B::MDEREF_HV_gvsv_vivify_rv2hv_helem()
-				) {
-					$ref = $sv->object_2svref;
-					bailout_multiref_vivify $S
-						if !$ref || ((ref($ref) eq 'SCALAR') && !$$ref);
-					$ref = $$ref;
 				} else {
 					bailout $S, "don't quite know what to do with multideref access=$access";
  				}
 			}
-
-			$reftype = (
-				$access == B::MDEREF_HV_padhv_helem() ||
-				$access == B::MDEREF_HV_gvsv_vivify_rv2hv_helem() ||
-				$access == B::MDEREF_HV_padsv_vivify_rv2hv_helem() ||
-        		        $access == B::MDEREF_HV_pop_rv2hv_helem() ||
-        		        $access == B::MDEREF_HV_vivify_rv2hv_helem() ||
-				$access == B::MDEREF_HV_gvhv_helem()
-			) ? 'HASH' : 'ARRAY';
  
 			my $key;
 			my $index = $actions & B::MDEREF_INDEX_MASK();
 
 			if ( $index != B::MDEREF_INDEX_none() ) {
 				if ( $index == B::MDEREF_INDEX_const() ) {
-					$key = ref($ptr) ? $ptr->object_2svref : $ptr;
+					$key = ${$ptr->object_2svref};
 				} elsif ( $index == B::MDEREF_INDEX_padsv() ) {
- 					$key = aux_get_padsv($AUX, $ptr)->object_2svref;
+ 					$key  = aux_get_padsv($AUX, $ptr)->object_2svref;
 				} elsif ( $index == B::MDEREF_INDEX_gvsv() ) {
-					$key = ref($ptr) ? $ptr->object_2svref : $ptr;
+					$key = ${$ptr->object_2svref};
 				}
-
  				$ref = $$ref if ref($ref) =~ /REF|SCALAR/;
- 				$key = $$key if ref($key) =~ /REF|SCALAR/;
+ 				$ref = (
+ 					$access == B::MDEREF_HV_padhv_helem() ||
+ 					$access == B::MDEREF_HV_padsv_vivify_rv2hv_helem() ||
+                		        $access == B::MDEREF_HV_pop_rv2hv_helem() ||
+                		        $access == B::MDEREF_HV_vivify_rv2hv_helem() ||
+					$access == B::MDEREF_HV_gvhv_helem()
+				) ? $ref->{$key} : $ref->[$key];
 
-				bailout $S, "Can't use value (\"$ref\") as an $reftype ref"
-					unless $reftype eq ref($ref);
-				if ( $reftype eq 'ARRAY') {
-					$ref = $ref->[$key];
-				} else {
-					$ref = $ref->{$key};
-				}
-
-				if (!defined($ref) && (
+				if (!$ref && (
 					$access == B::MDEREF_AV_gvsv_vivify_rv2av_aelem () ||
 					$access == B::MDEREF_AV_padsv_vivify_rv2av_aelem() ||
 					$access == B::MDEREF_AV_vivify_rv2av_aelem      () ||
@@ -469,18 +442,6 @@ sub get_value
 	} elsif ( $p{eval} && is_unop_aux($op, "multiconcat")) {
 		my @terms = parse_multiconcat($S, $op, eval => 1) or goto BAILOUT;
 		$val = join('', map { $_->{str} } @terms);
-	} elsif (is_op($op, "aelemfast_lex")) {
-		my $sv = $S->{padlist}->[1]->ARRAYelt($op->targ);
-		goto BAILOUT unless $sv;
-		$sv = $sv->object_2svref;
-		$val = $sv->[$op->private];
-	} elsif (is_svop($op, "aelemfast") || is_padop($op, "aelemfast")) {
-		my $sv = is_padop($op, "aelemfast") ?
-			$S->{padlist}->[1]->ARRAYelt($op->padix) :
-			$op->sv or goto BAILOUT;
-		$sv = $sv->object_2svref or goto BAILOUT;
-		$sv = ${$sv} or goto BAILOUT;
-		$val = $sv->[$op->private];
 	} else {
 	BAILOUT:
 		return () if $p{soft};
@@ -784,7 +745,7 @@ sub parse_term
 			("$term is not null", "$term is null") :
 			"$term is not null";
 	} elsif (($placeholder) = get_value($S, $op, soft => 1)) {
-		return 'null' if !defined($placeholder) && $S->{gen_args}->{inline};
+		return 'null' unless defined $placeholder;
 		goto PLACEHOLDER;
 	} elsif (is_unop($op, "backtick")) {
 		my $fop = $op->first;
@@ -912,26 +873,17 @@ sub get_gv
 	if (!$gv || !$$gv) {
 		$gv = $S->{padlist}->[1]->ARRAYelt($gv_idx);
 	}
-	if ( $p{get_name} && $gv->isa("B::IV")) {
-		my $subref = $gv->object_2svref;
-		if (ref($subref) eq 'REF' && ref($$subref) eq 'CODE') {
-			my $cv = B::svref_2object($$subref);
-			return $cv->NAME_HEK;
-		}
-	}
 	goto BAIL_OUT unless $gv->isa("B::GV");
-	return $p{get_name} ? $gv->NAME : $gv;
+	return $gv;
 BAIL_OUT:
 	bailout $S, "unable to get GV from \"", $op->name, "\"" if $p{bailout};
 	return;
 }
 
-sub  get_gv_name { get_gv(@_, get_name => 1) }
-
-sub try_get_subselect
+sub try_get_dbfetch
 {
 	my ($S, $sub) = @_;
-
+		
 	return unless is_unop($sub, "entersub");
 	$sub = $sub->first if is_unop($sub->first, "null");
 	return unless is_pushmark_or_padrange($sub->first);
@@ -950,7 +902,9 @@ sub try_get_subselect
 
 	$dbfetch = $dbfetch->first if is_unop($dbfetch->first, "null");
 	$dbfetch = $dbfetch->first;
-	return unless (get_gv_name($S, $dbfetch) // '') eq 'subselect';
+	my $gv = get_gv($S, $dbfetch);
+	return unless $gv;
+	return unless $gv->NAME eq 'subselect';
 
 	return $codeop;
 }
@@ -976,36 +930,6 @@ sub try_parse_funcall
 	return $sql;
 }
 
-sub in_list
-{
-	my ( $S, $sop, $list ) = @_;
-	$list //= [];
-
-	my $sql = '';
-	my $left = parse_term($S, $sop->first, not_after => 1);
-	return '1=0' unless @$list;
-
-	my $arg_limit = 
-		$S->{gen_args}->{in_arg_limit} //
-		((($S->{gen_args}->{flavor}||"") eq "oracle") ? 500 : 2_000_000_000);
-	my @args = @$list;
-	while ( @args ) {
-		my @placeholders;
-		for my $val ( splice(@args, 0, $arg_limit) ) {
-			if (( ref($val) // '') =~ /SCALAR/) {
-				push @placeholders, $$val;
-			} else {
-				push @placeholders, '?';
-				push @{$S->{values}}, $val;
-			}
-		}
-		$sql .= $left =~ / not$/ ? ' and ' : ' or ' if length $sql;
-		$sql .= "$left in (" . join(',', @placeholders) . ")";
-	}
-
-	return $sql;
-}
-
 sub try_parse_subselect
 {
 	my ($S, $sop) = @_;
@@ -1016,10 +940,14 @@ sub try_parse_subselect
 
 	if (is_op($sub, "padav")) {
 		my $ary = get_padlist_scalar($S, $sub->targ, "ref only");
-		return in_list( $S, $sop, $ary);
+		return '1=0' unless $ary && @$ary;
+		$sql = join ",", ("?") x @$ary;
+		@vals = @$ary;
 	} elsif (is_unop($sub, "rv2av") && is_op($sub->first, "padsv")) {
 		my $ary = get_padlist_scalar($S, $sub->first->targ, "ref only");
-		return in_list( $S, $sop, ${ $ary // \[] });
+		return '1=0' unless $ary && $$ary && @$$ary;
+		$sql = join ",", ("?") x @$$ary;
+		@vals = @$$ary;
 	} elsif (is_listop($sub, "anonlist") or
 			 is_unop($sub, "srefgen") &&
 			 is_unop($sub->first, "null") &&
@@ -1030,22 +958,24 @@ sub try_parse_subselect
 		for my $v (get_all_children($alist)) {
 			next if is_pushmark_or_padrange($v);
 			if (my ($const,$sv) = is_const($S, $v)) {
-				if (
-					($sv->isa("B::IV") && !$sv->isa("B::PVIV")) ||
-					($sv->isa("B::NV") && !$sv->isa("B::PVNV"))
-				) {
-					push @what, \$const;
-				} else {
+				if (($sv->isa("B::IV") && !$sv->isa("B::PVIV")) ||
+					($sv->isa("B::NV") && !$sv->isa("B::PVNV")))
+				{
 					push @what, $const;
+				} else {
+					push @what, '?';
+					push @vals, $const;
 				}
 			} else {
 				my ($val, $ok) = get_value($S, $v);
-				push @what, $val;
+				push @what, '?';
+				push @vals, $val;
 			}
 		}
-		return in_list($S, $sop, \@what);
+		bailout $S, "empty list in not valid in \"<-\"" unless @what;
+		$sql = join ",", @what;
 	} else {
-		my $codeop = try_get_subselect( $S, $sub);
+		my $codeop = try_get_dbfetch( $S, $sub);
 		if ($codeop) {
 			$sql = handle_subselect($S, $codeop);
 		} else {
@@ -1103,7 +1033,7 @@ sub parse_assign
 			my $tab = try_parse_attr_assignment($S,
 				$op->last->first->sibling, $val);
 			return if $tab;
-		} elsif (my $codeop = try_get_subselect($S, $op->first)) {
+		} elsif (my $codeop = try_get_dbfetch($S, $op->first)) {
 			my $sql = handle_subselect($S, $codeop, returns_dont_care => 1);
 			my $tab = try_parse_attr_assignment($S,
 				$op->last->first->sibling, "($sql)");
@@ -1168,8 +1098,9 @@ sub try_funcall
 		return unless @args;
 		$op = pop @args;
 		return unless is_svop($op, "gv") || is_padop($op, "gv");
-		my $func = get_gv_name( $S, $op);
-		return unless defined $func;
+		my $gv = get_gv($S, $op);
+		return unless $gv;
+		my $func = $gv->NAME;
 		${$p{func_name_return}} = $func if $p{func_name_return};
 		if ($func =~ /^(union|intersect|except)$/) {
 			return if $p{only_normal_funcs};
@@ -1251,10 +1182,6 @@ sub try_funcall
 				@terms = ("$val from $terms[1]");
 			}
 		}
-		if (lc($func) eq 'cast' && @terms == 2) {
-			$terms[1] = $terms[1]->undo if UNIVERSAL::isa($terms[1], "DBIx::Perlish::Placeholder");
-			return "cast($terms[0] as $terms[1])";
-		}
 		return "$func(" . join(", ", @terms) . ")";
 	}
 }
@@ -1319,8 +1246,8 @@ sub parse_multi_assign
 		$op = $op->first if is_unop($op, "null");
 		$op = $op->sibling if is_pushmark_or_padrange($op);
 		$op = $op->first if is_unop($op, "rv2cv");
-		my $gv = get_gv_name($S, $op);
-		$tab = $gv if defined $gv;
+		my $gv = get_gv($S, $op);
+		$tab = $gv->NAME if $gv;
 	}
 	bailout $S, "cannot get a table to update" unless $tab;
 }
@@ -1544,10 +1471,10 @@ sub try_special_concat
 	@terms = grep { !$_->{skip} } @terms;
 	while (@terms) {
 		my $t = shift @terms;
-		if (exists $t->{str}) {
+		if ($t->{str}) {
 			$str .= $t->{str};
-		} elsif (exists $t->{field}) {
-			if (length($str)) {
+		} elsif ($t->{field}) {
+			if ($str) {
 				push @v, $str;
 				push @sql, '?';
 			}
@@ -1560,10 +1487,10 @@ sub try_special_concat
 		} else {
 			my $t2 = shift @terms;
 			return () unless $t2;
-			return () unless defined $t2->{str};
+			return () unless $t2->{str};
 			return () unless $t2->{str} =~ s/^->(\w+)//;
 			my $f = $1;
-			if (length($str)) {
+			if ($str) {
 				push @v, $str;
 				push @sql, '?';
 			}
@@ -1575,7 +1502,7 @@ sub try_special_concat
 			$str = $t2->{str};
 		}
 	}
-	if (length($str)) {
+	if ($str) {
 		push @v, $str;
 		push @sql, '?';
 	}
@@ -1622,7 +1549,6 @@ sub parse_complex_regex
 	} elsif ( is_svop( $op, 'const')) {
 		return want_const( $S, $op);
 	} elsif (my ($rx, $ok) = get_value($S, $op, soft => 1)) {
-		return undef unless $rx;
 		$rx =~ s/^\(\?\-\w*\:(.*)\)$/$1/; # (?-xism:moo) -> moo
 		return $rx;
 	} elsif (is_unop($op, "null")) {
@@ -1649,7 +1575,6 @@ sub parse_regex
 		bailout $S, "strange regex " . $op->name
 			unless $logop and is_logop( $logop, 'regcomp');
 		$like = parse_complex_regex( $S, $logop-> first);
-		return "" unless defined $like; # explicitly nulled like
 	}
 
 	my $lhs = parse_term($S, $op->first);
@@ -1657,18 +1582,16 @@ sub parse_regex
 	my $flavor = lc($S-> {gen_args}-> {flavor} || '');
 	my $what = 'like';
 
-	$like =~ s/\(\?\^\w+\:((?:[^\\]|\\.)*)\)/$1/g; # ignore ?^flags:
-
-	my $can_like = $like =~ /^\^?(?:[-!%\s\w]|\\.)*\$?$/; # like that begins with non-% can use indexes
-
+	my $can_like = $like =~ /^\^?[-!%\s\w]*\$?$/; # like that begins with non-% can use indexes
+	
 	if ( $flavor eq 'mysql') {
 		# mysql LIKE is case-insensitive
 		goto LIKE if not $case and $can_like;
 		$like =~ s/'/''/g;
 
-		return
+		return 
 			"$lhs ".
-			( $neg ? 'not ' : '') .
+			( $neg ? 'not ' : '') . 
 			'regexp ' .
 			( $case ? '' : 'binary ') .
 			"'$like'"
@@ -1678,11 +1601,11 @@ sub parse_regex
 		if ( $can_like) {
 			$what = 'ilike' if $case;
 			goto LIKE;
-		}
+		} 
 		$like =~ s/'/''/g;
-		return
+		return 
 			"$lhs ".
-			( $neg ? '!' : '') .
+			( $neg ? '!' : '') . 
 			'~' .
 			( $case ? '*' : '') .
 			" '$like'"
@@ -1718,6 +1641,7 @@ sub parse_regex
 		# return "$lhs $what ?";
 		return ($neg ? "not " : "") . "$what(?, $lhs)";
 	} else {
+		$like =~ s/'/''/g;
 		# XXX is SQL-standard LIKE case-sensitive or not?
 		if ($case) {
 			$lhs = "lower($lhs)";
@@ -1726,15 +1650,12 @@ sub parse_regex
 		bailout $S, "Regex too complex for implementation using LIKE keyword: $like"
 			if $like =~ /(?<!\\)[\[\]\(\)\{\}\?\|]/;
 LIKE:
-		$like =~ s/'/''/g;
-		$like =~ s/\\([^A-Za-z_0-9])/$1/g; # de-quotemeta
 		my $escape = "";
 		if ($flavor eq "pg" || $flavor eq "oracle") {
 			# XXX it is possible that more flavors support like...escape
-			my $need_esc;
-			$need_esc = 1 if $like =~ s/!/!!/g;
-			$need_esc = 1 if $like =~ s/%/!%/g;
-			$need_esc = 1 if $like =~ s/_/!_/g;
+			my $need_esc = 1 if $like =~ s/!/!!/g;
+			   $need_esc = 1 if $like =~ s/%/!%/g;
+			   $need_esc = 1 if $like =~ s/_/!_/g;
 			$escape = " escape '!'" if $need_esc;
 		} else {
 			$like =~ s/%/\\%/g;
@@ -1745,8 +1666,8 @@ LIKE:
 		$like =~ s/\\\././g;
 		$like = "%$like" unless $like =~ s|^\^||;
 		$like = "$like%" unless $like =~ s|\$$||;
-		return "$lhs " .
-			( $neg ? 'not ' : '') .
+		return "$lhs " . 
+			( $neg ? 'not ' : '') . 
 			"$what '$like'$escape"
 		;
 	}
@@ -1811,7 +1732,7 @@ sub parse_join
 	# subselect
 	my ( $condition, $codeop);
 	if ( $op[2]) {
-		$codeop = try_get_subselect( $S, $op[2]);
+		$codeop = try_get_dbfetch( $S, $op[2]);
 		bailout $S, "third argument to join is not a subselect expression"
 			unless $codeop;
 
@@ -2264,8 +2185,7 @@ if (*Devel::Cover::coverage{CODE}) {
 	$_cover = sub { $Coverage->{statement}{Devel::Cover::get_key($_[0])} ||= 1 };
 }
 
-package
-	DBIx::Perlish::Placeholder;
+package DBIx::Perlish::Placeholder;
 
 use overload '""' => sub { "?" }, eq => sub { "$_[0]" eq "$_[1]" };
 
